/* sqfigbut.h -- Square-Figure (B version) utilities.
**
** Copyright (C) 2003 Eric Laroche.  All rights reserved.
**
** @author Eric Laroche <laroche@lrdev.com>
** @version @(#)$Id: sqfigbut.h,v 1.2 2003/10/05 10:06:50 laroche Exp $
**
** @reference http://www.lrdev.com/lr/c/sqfig.html
**
** This program is free software;
** you can redistribute it and/or modify it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*/


#ifndef SQFIGBUT_H
#define SQFIGBUT_H


/** Nth bit.
* Note: macro guarantees single evaluation.
*/
#define BIT(n) (1 << (n))

/** Check nth bit.
* Note: macro guarantees single evaluation.
*/
#define BITSETP(i, n) (((i) & BIT((n))) != 0)

/** Maximal number of bits allowed.
* Assuming eight bit bytes.
*/
#define MAXBITS (8 * sizeof(int))

/** Bits to (but excluding) nth bit.
* Note: macro guarantees single evaluation.
* However >>MAXBITS may turn out to be a null operation, so BITSTO_(0)
* will result in a wrong value.
*/
#define BITSTO_(n) ((unsigned int)~0 >> (MAXBITS - (n)))

/** Bits to (but excluding) nth bit.
* Note: macro evaluates n twice!
*/
#define BITSTO(n) ((n) > 0 ? BITSTO_((n)) : 0)

/** Bits from (including) ith bit to (but excluding) nth bit.
* Note: macro evaluates n twice by use of BITSTO!
* However <<MAXBITS may turn out to be a null operation, so
* BITSFROMTO_(MAXBITS,MAXBITS) will result in a wrong value.
*/
#define BITSFROMTO_(i, n) ((~0 << (i)) & BITSTO((n)))

/** Bits from (including) ith bit to (but excluding) nth bit.
* Note: macro evaluates i and n twice!
*/
#define BITSFROMTO(i, n) ((i) < MAXBITS ? BITSFROMTO_((i), (n)): 0)

/** Check for any nth bit in a vector.
*/
int bitSetVectorP(int const* vector, int size, int n);

/** Initialize a vector.
*/
void initVector(int* vector, int size);

/** Find first bit.
* Returns -1 if none is found.
*/
int firstBit(int i);

/** Find first bit in a vector.
*/
void firstBitVector(int const* vector, int ySize, int* y, int* x);

/** Adjust dimensions.
*/
void adjustDimensions(int const* bitmap, int* ySize, int* xSize);


#endif

